/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2025 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.customoperations;

import ca.uhn.fhir.jpa.api.dao.DaoRegistry;
import ca.uhn.fhir.jpa.model.util.JpaConstants;
import ca.uhn.fhir.model.api.annotation.Description;
import ca.uhn.fhir.rest.annotation.IdParam;
import ca.uhn.fhir.rest.annotation.Operation;
import ca.uhn.fhir.rest.annotation.OperationParam;
import ca.uhn.fhir.rest.api.server.RequestDetails;
import jakarta.servlet.http.HttpServletRequest;
import org.hl7.fhir.instance.model.api.IBaseBundle;
import org.hl7.fhir.r4.model.Bundle;
import org.hl7.fhir.r4.model.IdType;
import org.hl7.fhir.r4.model.IntegerType;
import org.hl7.fhir.r4.model.Parameters;
import org.hl7.fhir.r4.model.Patient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Date;

public class CustomOperations {
	private static final Logger ourLog = LoggerFactory.getLogger(CustomOperations.class);
	private static final String TEST_BUNDLE_ID = "OUTPUT-TEST-BUNDLE-ID";
	public static final String Y2K_AGE_PARAMETER = "y2k-age";
	public static final String CALCULATE_Y2K_AGE_OPERATION_NAME = "calculate-y2k-age";

	@Autowired
	DaoRegistry myDaoRegistry;

	/**
	 * System-level /$process-message operation
	 */
	@Operation(name = JpaConstants.OPERATION_PROCESS_MESSAGE, idempotent = false)
	public IBaseBundle processMessage(
			HttpServletRequest theServletRequest,
			RequestDetails theRequestDetails,
			@OperationParam(name = "content", min = 1, max = 1)
					@Description(
							formalDefinition =
									"The message to process (or, if using asynchronous messaging, it may be a response message to accept)")
					Bundle theMessageToProcess) {
		// Below is a dummy implementation of the $process-message operation. Replace this code with the actual
		// implementation.
		Bundle retval = new Bundle();
		retval.setId(TEST_BUNDLE_ID);
		ourLog.info("PROCESSING MESSAGE: {}", theMessageToProcess.getId());
		return retval;
	}

	@Operation(name = CALCULATE_Y2K_AGE_OPERATION_NAME, type = Patient.class, idempotent = true)
	public Parameters calculateY2KAge(
			HttpServletRequest theServletRequest,
			RequestDetails theRequestDetails,
			@Description(formalDefinition = "Operation that calculates the age of a patient on Jan 1, 2000") @IdParam
					IdType thePatientId) {

		ourLog.info("PROCESSING {}", thePatientId.getValueAsString());
		Patient patient = myDaoRegistry.getResourceDao(Patient.class).read(thePatientId);

		Parameters retval = new Parameters();
		if (patient.getBirthDate() != null) {
			long jan1y2k = Date.valueOf("2000-01-01").getTime();
			long age = (jan1y2k - patient.getBirthDate().getTime()) / (1000L * 60 * 60 * 24 * 365);
			retval.addParameter().setName(Y2K_AGE_PARAMETER).setValue(new IntegerType((int) age));
		}
		return retval;
	}
}
